// 1RunIt.sce

//*****************************************************************************
// Programa de evaluación y pruebas aleatorias automatizadas de
// los algoritmos de estimación de orientación para vehículos aéreos.
// Autores: Yuriy Kovalenko, Gilberto Bustamante Balderas, Pedro Huerta Durán
//*****************************************************************************

clc;
clear;
xdel(winsid());

exec("TestParam.sce");
exec("SimParam.sce");
exec("FrameParam.sce");
exec("SensParam.sce");
exec("StateEstimat.sce");
exec("trim.sce");
exec("Sensors.sce");
//exec("MDNonLin.sce");
exec("MDLin.sce");
exec("Geometry.sce");
exec("Animation.sce");
exec("FigAndFiles.sce");
exec("Comparator.sce");

BestY0=0;// Init. cond. resulted in max error 
BestDelta=0; // Controls resulted in max error 

rand("seed",32);

MaxErr=0;
TestNumb=10;// Number of trials 

tInit=0;
tFin=1.5*60; // segundos
step=0.2; // segundos
t=tInit:step:tFin;

NoiseLevel=0; //Sensors noise level (0 or 1)

//Va=30;
//Gamma=10*%pi/180;
//global R
////If R=0 or R=%inf => wings level flight
////R=%inf;//Radius=infinitum
////R=0;//Radius=infinitum
//R=300;
//TrimPar=[Va;Gamma;R];
//
// valores iniciales
//pn0=0;// Inertial north position of the MAV en F^i
//pe0=0;// Inertial east position of the MAV en F^i
//pd0=0;// Inertial down position (negative of altitude) of the MAV measured en F^i
//u0=Va;// Body frame velocity measured along i^b in F^b
//v0=0; // Body frame velocity measured along j^b in F^b
//w0=0; // Body frame velocity measured along k^b in F^b
//phi0=0; // Roll angle defined with respect to F^v2
//theta0=Gamma; // Pitch angle defined with respect to F^v1
//psi0=0; // Heading (yaw) angle defined with respect to F^v
//p0=0; // Roll rate measured en i^b in F^b
//q0=0; // Pitch rate measured along j^b in F^b
//r0=0; // Yaw rate measured along k^b in F^b
//
//y0=[pn0;pe0;pd0;u0;v0;w0;phi0;theta0;psi0;p0;q0;r0];
//
// variables de control
//delta_a=0; // aileron
//delta_e=0*%pi/180; // elevator
//delta_r=0; // rudder
//delta_t=1; // thrust
//delta=[delta_a;delta_e;delta_r;delta_t];
//

tic();

//Initial condition generator
for iter=1:TestNumb
	printf("\nIteration %d\n",iter);
	// initial conditions
	y0=xBound(:,1)+(xBound(:,2)-xBound(:,1)).*rand(12,1);
	// UAV controls
	delta=deltaBound(:,1)+(deltaBound(:,2)-deltaBound(:,1)).*rand(4,1);


	// Trimming
	//[yTrimmed,deltaTrimmed]=trim(TrimPar);
	//disp(yTrimmed,"yTrimmed");
	//disp(deltaTrimmed,"deltaTrimmed");
	//y0=yTrimmed;
	//delta=deltaTrimmed;
	//xDotCurrent=DynMod(0,y0,delta);
	//disp(xDotCurrent',"xDot after trimming");
	//
	// Flight simulation
	MDResult=DynamicModel(y0, tInit, t, delta);
	//printf("Tiempo de calculo: %f\n",toc());
//	Graph(1,WTitles(1),subWtitles(1:6),[t;MDResult(1:6,:)]);
//	Graph(2,WTitles(2),subWtitles(7:12),[t;MDResult(7:12,:)]);
	//Save2File([t',MDResult'],DMResultFile,DMFileHeader);

	// Sensor signal generation
	SensResult=SensorSimul();
//	Graph(3,WTitles(3),subWtitles(13:18),[t;SensResult(1:6,:)]);
//	Graph(4,WTitles(4),subWtitles(19:23),[t;SensResult(7:11,:)]);
	//Save2File([t',SensResult'],SensResFile,SenFileHeader);

	// State estimation
	StateEst=stateEstimation();
//	Graph(5,WTitles(5),subWtitles(1:6),[t;StateEst(1:6,:)]);
//	Graph(6,WTitles(6),subWtitles(7:12),[t;StateEst(7:12,:)]);
	//Save2File([t',StateEst'],StateEstFile,DMFileHeader);

	// Animation
	PosAng=[MDResult(1:3,:);MDResult(7:9,:)];
	DrawTrajectory(PosAng,7);

	[MaxVal,Val]=comparator(MDResult(7:12,:),StateEst(7:12,:));
	printCompRes(MaxVal,Val);
	if(MaxVal>MaxErr)
		MaxErr=MaxVal
		printf('\nIter=%d MaxErr= %f\n',iter,MaxErr);
		BestY0=y0;// Init. cond. resulted in max error 
  BestDelta=delta; // Controls resulted in max error 
//		printf('Err= %f\n',Val);  
//		printf('Init Val x0= %f\n',y0);  
//		printf('Controls (deltas)= %f\n',delta);  
	end  
end
printf("execution time=%g s\n",toc());

 y0=BestY0;// Init. cond. resulted in max error 
 delta=BestDelta; // Controls resulted in max error 

	// Flight simulation
	MDResult=DynamicModel(y0, tInit, t, delta);
	Graph(1,WTitles(1),subWtitles(1:6),[t;MDResult(1:6,:)]);
	Graph(2,WTitles(2),subWtitles(7:12),[t;MDResult(7:12,:)]);
	Save2File([t',MDResult'],DMResultFile,DMFileHeader);

	// Sensor signal generation
	SensResult=SensorSimul();
	Graph(3,WTitles(3),subWtitles(13:18),[t;SensResult(1:6,:)]);
	Graph(4,WTitles(4),subWtitles(19:23),[t;SensResult(7:11,:)]);
 Save2File([t',SensResult'],SensResFile,SenFileHeader);

	// State estimation
	StateEst=stateEstimation();
	Graph(5,WTitles(5),subWtitles(1:6),[t;StateEst(1:6,:)]);
	Graph(6,WTitles(6),subWtitles(7:12),[t;StateEst(7:12,:)]);
 Save2File([t',StateEst'],StateEstFile,DMFileHeader);

	// Animation
	PosAng=[MDResult(1:3,:);MDResult(7:9,:)];
	DrawTrajectory(PosAng,7);
	[MaxVal,Val]=comparator(MDResult(7:12,:),StateEst(7:12,:));
	printCompRes(MaxVal,Val);
	printf('\nMaxErr= %f\n',MaxVal);
	disp(y0,"Initial value");
	disp(delta,"Initial controls values");
