// MDLin.sce

//*****************************************************************************
// Programa de evaluación y pruebas aleatorias automatizadas de
// los algoritmos de estimación de orientación para vehículos aéreos.
// Autores: Yuriy Kovalenko, Gilberto Bustamante Balderas, Pedro Huerta Durán
//*****************************************************************************

//Modelo Dinamico Lineal de Aerosonde

function ydot = DynMod(t,y,delta)
   // variables de control
  delta_a=delta(1); // aileron
  delta_e=delta(2); // elevator
  delta_r=delta(3); // rudder
  delta_t=delta(4); // trust

  Vel=y(4:6); // velocidades en body frame
  
  phi=y(7); // Roll angle defined with respect to F^v2
  theta=y(8); // Pitch angle defined with respect to F^v1
  psi=y(9); // Heading (yaw) angle defined with respect to F^v
  V_a=norm(Vel,2);
  alpha=atan(y(6),y(4));
  Beta=asin(y(5)/V_a);
  VelAng=y(10:12); // [p,q,r] velocidades angulares
  p=y(10); q=y(11); r=y(12);
  
  Sphi=sin(phi); Cphi=cos(phi);
  Stheta=sin(theta); Ctheta=cos(theta);
  Spsi=sin(psi); Cpsi=cos(psi);
  Ttheta=tan(theta);
  Calpha=cos(alpha); Salpha=sin(alpha);
  Cbeta=cos(Beta); Sbeta=sin(Beta);
  
  Rvtov1=[Cpsi,Spsi,0;-Spsi,Cpsi,0;0,0,1];
  Rv1tov2=[Ctheta,0,-Stheta;0,1,0;Stheta,0,Ctheta];
  Rv2tob=[1,0,0;0,Cphi,Sphi;0,-Sphi,Cphi];
  Rvtob=Rv2tob*Rv1tov2*Rvtov1;
  Rbtos=[Calpha,0,Salpha;0,1,0;-Salpha,0,Calpha]; 
  Rstob=Rbtos';
  R33=[1,Sphi*Ttheta,Cphi*Ttheta;
       0,Cphi,-Sphi;
       0,Sphi/Ctheta,Cphi/Ctheta];
  // fuerzas 
  f_g=Rvtob*[0;0;P.m*P.g];
  f_p=1/2*P.rho*P.S_prop*P.C_prop*[(P.k_motor*delta_t)^2-V_a^2;0;0];
  
  C_L=P.C_L_0+P.C_L_alpha*alpha;
  C_D=P.C_D_0+P.C_D_alpha*alpha;
  
  Coeff=1/2*P.rho*V_a^2*P.S;
  F_drag=Coeff*(C_D+P.C_D_q*P.c*q/(2*V_a)+P.C_D_delta_e*delta_e);

  f_y=Coeff*(P.C_Y_0+P.C_Y_Beta*Beta+P.C_Y_p*P.b*p/(2*V_a)+...
      P.C_Y_r*P.b*r/(2*V_a)+P.C_Y_delta_a*delta_a+P.C_Y_delta_r*delta_r);
  F_lift=Coeff*(C_L+P.C_L_q*P.c*q/(2*V_a)+P.C_L_delta_e*delta_e);
  f_a=Rstob*[-F_drag;f_y;-F_lift];
  fuerzas=f_g + f_a + f_p;
  
//  Momentos
  m_p=[-P.k_T_p*(P.k_omega*delta_t)^2;0;0];
  l=Coeff*P.b*(P.C_l_0+P.C_l_Beta*Beta+P.C_l_p*P.b*p/(2*V_a)...
    +P.C_l_r*P.b*r/(2*V_a)+P.C_l_delta_a*delta_a+P.C_l_delta_r*delta_r);
  m=Coeff*P.c*(P.C_m_0+P.C_m_alpha*alpha+P.C_m_q*P.c*q/(2*V_a)...
    +P.C_m_delta_e*delta_e);
  n=Coeff*P.b*(P.C_n_0+P.C_n_Beta*Beta+P.C_n_p*P.b*p/(2*V_a)...
    +P.C_n_r*P.b*r/(2*V_a)+P.C_n_delta_a*delta_a+P.C_n_delta_r*delta_r);
  m_a=[l;m;n];
  momentos=m_a+m_p;
  
  // dinámica
  dotPos=Rvtob'*Vel;
  dotVel=fuerzas/P.m-cross(VelAng,Vel);
  dotAng=R33*VelAng;
  dotVelAng=inv(P.J)*(momentos-cross(VelAng,(P.J*VelAng)))

  ydot=[dotPos;dotVel;dotAng;dotVelAng];
endfunction

function y =DynamicModel(y0, tInit, t, delta)
  y = ode(y0, tInit, t, list(DynMod,delta));
endfunction
