// StateEstimat.sce

//*****************************************************************************
// Programa de evaluación y pruebas aleatorias automatizadas de
// los algoritmos de estimación de orientación para vehículos aéreos.
// Autores: Yuriy Kovalenko, Gilberto Bustamante Balderas, Pedro Huerta Durán
//*****************************************************************************


function EstRes=stateEstimation()
  // reading sensor signals data
  //SensResults = fscanfMat(SensResFile); 
  //Time accelX accelY accelZ gyroX gyroY gyroZ PressAbs PressDiff GPS.X 
  //GPS.Y GPS.Z

  accel=SensResult(1:3,:);// accel x, y, z
  gyro=SensResult(4:6,:);// gyro x, y, z
  PressAbs=SensResult(7,:);
  PressDiff=SensResult(8,:);
  gps=SensResult(9:11,:);// GPS x, y, z
  
  n=length(t);
  ePos=zeros(3,n);//estimated P_north,P_east,P_down 
  eVel=zeros(3,n);//estimated u,v,w 
  eVelPred=zeros(3,n);//Predicted estimated u,v,w 
  eAng=zeros(3,n);//estimated phi,theta,psi
  eAngRate=zeros(3,n);//estimated p,q,r
  eAbsPres=zeros(1,n);
  Altit=zeros(1,n);

  //Initial values
  ePos(:,1)=MDResult(1:3,1);//estimated P_north,P_east,P_down  
  eVel(:,1)=MDResult(4:6,1);//u,v,w    
  eVelPred(:,1)=MDResult(4:6,1);//Predicted u,v,w     
  eAng(:,1)=MDResult(7:9,1);//estimated phi,theta,psi
  eAngRate(:,1)=MDResult(10:12,1);// p,q,r
  eAbsPres(1,1)=P.Pstd;
  Altit(1,1)=-pd0;

  alphGyrX=0.5; alphGyrY=0.5; alphGyrZ=0.5; 
  alphGyr=[alphGyrX;alphGyrY;alphGyrZ]
  alphAbsPres=0.5;

  dt=t(2)-t(1);// time step

  // Filtration
  for i=1:n-1
    eAngRate(:,i+1)=alphGyr.*eAngRate(:,i)+(1-alphGyr).*gyro(:,i);
    eAbsPres(i+1)=alphAbsPres*eAbsPres(i)+(1-alphAbsPres)*PressAbs(i);
    //  Altit(i+1)=-pd0-eAbsPres(i+1)/P.rho/P.g;
    Altit(1,i+1)=-(eAbsPres(i+1)-P.Pstd)/P.rho/P.g;
  end

  //Integration (Heun method)
  for i=1:n-1 //integration
    alpha=atan(eVel(3,i),eVel(1,i));
    V_a=norm(eVel(:,i),2);
    Beta=asin(eVel(2,i)/V_a);
    Sphi=sin(eAng(1,i)); Cphi=cos(eAng(1,i));
    Stheta=sin(eAng(2,i)); Ctheta=cos(eAng(2,i));
    Spsi=sin(eAng(3,i)); Cpsi=cos(eAng(3,i));
    Ttheta=tan(eAng(2,i));
    Calpha=cos(alpha); Salpha=sin(alpha);
    Cbeta=cos(Beta); Sbeta=sin(Beta);
    Rvtov1=[Cpsi,Spsi,0;-Spsi,Cpsi,0;0,0,1];
    Rv1tov2=[Ctheta,0,-Stheta;0,1,0;Stheta,0,Ctheta];
    Rv2tob=[1,0,0;0,Cphi,Sphi;0,-Sphi,Cphi];
    Rvtob=Rv2tob*Rv1tov2*Rvtov1;
    R33=[1,Sphi*Ttheta,Cphi*Ttheta;
         0,Cphi,-Sphi;
         0,Sphi/Ctheta,Cphi/Ctheta];
//    GravAccel=P.g*[-Stheta; Sphi*Ctheta; Cphi*Ctheta];
    GravAccel=0;
    eAng(:,i+1)=eAng(:,i)+dt/2.*R33*(eAngRate(:,i)+eAngRate(:,i+1));
    
//    eVel(:,i+1)=Rvtob'*(accel(:,i)-a_g);
    //eVel(:,i+1)=eVel(:,i)+(accel(:,i)-a_g).*dt
    eVel(:,i+1)=eVel(:,i)+(accel(:,i)-GravAccel-cross(eAngRate(:,i),eVel(:,i))).*dt
//    eVel(:,i+1)=eVel(:,i)+dt/2.*(accel(:,i)-GravAccel-cross(eAngRate(:,i),eVel(:,i))+accel(:,i+1)-a_g-cross(eAngRate(:,i+1),eVelPred(:,i+1))); 
    ePos(:,i+1)=ePos(:,i)+dt/2.*Rvtob'*(eVel(:,i)+eVel(:,i+1));
  end
  EstRes=[ePos(:,:);eVel(:,:);eAng(:,:);eAngRate(:,:);Altit(:,:)];
endfunction
